# API Docs - v1.0.4

## Source

### http *<a target="_blank" href="https://siddhi.io/en/v4.x/docs/query-guide/#source">(Source)</a>*

<p style="word-wrap: break-word">The HTTP source receives POST requests via HTTP or HTTPS in format such as <code>text</code>, <code>XML</code> and <code>JSON</code>. If required, you can enable basic authentication to ensure that events are received only from users who are authorized to access the service.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
@source(type="http", receiver.url="<STRING>", basic.auth.enabled="<STRING>", worker.count="<STRING>", @map(...)))
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">receiver.url</td>
        <td style="vertical-align: top; word-wrap: break-word">The URL to which the events should be received. User can provide any valid url and if the url is not provided the system will use the following format <code>http://0.0.0.0:9763/&lt;appNAme&gt;/&lt;streamName&gt;</code>If the user want to use SSL the url should be given in following format <code>https://localhost:8080/&lt;streamName&gt;</code></td>
        <td style="vertical-align: top">http://0.0.0.0:9763/<appNAme>/<streamName></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">basic.auth.enabled</td>
        <td style="vertical-align: top; word-wrap: break-word">If this is set to <code>true</code>, basic authentication is enabled for incoming events, and the credentials with which each event is sent are verified to ensure that the user is authorized to access the service. If basic authentication fails, the event is not authenticated and an authentication error is logged in the CLI. By default this values 'false' </td>
        <td style="vertical-align: top">false</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">worker.count</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of active worker threads to serve the incoming events. The value is 1 by default. This will ensure that the events are directed to the event stream in the same order in which they arrive. By increasing this value the performance might increase at the cost of loosing event ordering.</td>
        <td style="vertical-align: top">1</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="system-parameters" class="md-typeset" style="display: block; font-weight: bold;">System Parameters</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Parameters</th>
    </tr>
    <tr>
        <td style="vertical-align: top">latency.metrics.enabled</td>
        <td style="vertical-align: top; word-wrap: break-word">Property to enable metrics logs to monitor transport latency for config.</td>
        <td style="vertical-align: top">true</td>
        <td style="vertical-align: top">true<br>false</td>
    </tr>
    <tr>
        <td style="vertical-align: top">server.bootstrap.socket.timeout</td>
        <td style="vertical-align: top; word-wrap: break-word">property to configure specified timeout in milliseconds which server socket will block for this amount of time for http message content to be received.</td>
        <td style="vertical-align: top">15</td>
        <td style="vertical-align: top">Any integer</td>
    </tr>
    <tr>
        <td style="vertical-align: top">server.bootstrap.boss.group.size</td>
        <td style="vertical-align: top; word-wrap: break-word">property to configure number of boss threads, which accepts incoming connections until the ports are unbound. Once connection accepts successfully, boss thread passes the accepted channel to one of the worker threads.</td>
        <td style="vertical-align: top">4</td>
        <td style="vertical-align: top">Any integer</td>
    </tr>
    <tr>
        <td style="vertical-align: top">server.bootstrap.worker.group.size</td>
        <td style="vertical-align: top; word-wrap: break-word">property to configure number of worker threads, which performs non blocking read and write for one or more channels in non-blocking mode.</td>
        <td style="vertical-align: top">8</td>
        <td style="vertical-align: top">Any integer</td>
    </tr>
    <tr>
        <td style="vertical-align: top">default.host</td>
        <td style="vertical-align: top; word-wrap: break-word">The default host of the transport.</td>
        <td style="vertical-align: top">0.0.0.0</td>
        <td style="vertical-align: top">Any valid host</td>
    </tr>
    <tr>
        <td style="vertical-align: top">http.port</td>
        <td style="vertical-align: top; word-wrap: break-word">The default port if the default scheme is 'http'.</td>
        <td style="vertical-align: top">9763</td>
        <td style="vertical-align: top">Any valid port</td>
    </tr>
    <tr>
        <td style="vertical-align: top">https.port</td>
        <td style="vertical-align: top; word-wrap: break-word">The default port if the default scheme is 'https'.</td>
        <td style="vertical-align: top">9443</td>
        <td style="vertical-align: top">Any valid port</td>
    </tr>
    <tr>
        <td style="vertical-align: top">default.protocol</td>
        <td style="vertical-align: top; word-wrap: break-word">The default protocol.</td>
        <td style="vertical-align: top">http</td>
        <td style="vertical-align: top">http<br>https</td>
    </tr>
    <tr>
        <td style="vertical-align: top">https.keystore.file</td>
        <td style="vertical-align: top; word-wrap: break-word">The default keystore file path.</td>
        <td style="vertical-align: top">${carbon.home}/resources/security/wso2carbon.jks</td>
        <td style="vertical-align: top">Path to wso2carbon.jks file</td>
    </tr>
    <tr>
        <td style="vertical-align: top">https.keystore.password</td>
        <td style="vertical-align: top; word-wrap: break-word">The default keystore password.</td>
        <td style="vertical-align: top">wso2carbon</td>
        <td style="vertical-align: top">String of keystore password</td>
    </tr>
    <tr>
        <td style="vertical-align: top">https.cert.password</td>
        <td style="vertical-align: top; word-wrap: break-word">The default cert password.</td>
        <td style="vertical-align: top">wso2carbon</td>
        <td style="vertical-align: top">String of cert password</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@source(type='http', receiver.url='http://localhost:9055/endpoints/RecPro', @map(type='xml'))
define stream FooStream (symbol string, price float, volume long);

```
<p style="word-wrap: break-word">Above source configuration performs a default XML input mapping. The expected input is as follows:&lt;events&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/event&gt;<br>&lt;/events&gt;<br>If basic authentication is enabled via the <code>basic.auth.enabled='true</code> setting, each input event is also expected to contain the <code>Authorization:'Basic encodeBase64(username:Password)'</code> header.</p>

## Sink

### http *<a target="_blank" href="https://siddhi.io/en/v4.x/docs/query-guide/#sink">(Sink)</a>*

<p style="word-wrap: break-word">This extension publish the HTTP events in any HTTP method  POST, GET, PUT, DELETE  via HTTP or https protocols. As the additional features this component can provide basic authentication as well as user can publish events using custom client truststore files when publishing events via https protocol. And also user can add any number of headers including HTTP_METHOD header for each event dynamically.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
@sink(type="http", publisher.url="<STRING>", basic.auth.username="<STRING>", basic.auth.password="<STRING>", client.truststore.path="<STRING>", client.truststore.password="<STRING>", headers="<STRING>", method="<STRING>", @map(...)))
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">publisher.url</td>
        <td style="vertical-align: top; word-wrap: break-word">The URL to which the outgoing events should be published via HTTP. This is a mandatory parameter and if this is not specified, an error is logged in the CLI. If user wants to enable SSL for the events, use <code>https</code> instead of <code>http</code> in the publisher.url.e.g., <code>http://localhost:8080/endpoint</code>, <code>https://localhost:8080/endpoint</code></td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">basic.auth.username</td>
        <td style="vertical-align: top; word-wrap: break-word">The username to be included in the authentication header of the basic authentication enabled events. It is required to specify both username and password to enable basic authentication. If one of the parameter is not given by user then an error is logged in the CLI.</td>
        <td style="vertical-align: top"> </td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">basic.auth.password</td>
        <td style="vertical-align: top; word-wrap: break-word">The password to include in the authentication header of the basic authentication enabled events. It is required to specify both username and password to enable basic authentication. If one of the parameter is not given by user then an error is logged in the CLI.</td>
        <td style="vertical-align: top"> </td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">client.truststore.path</td>
        <td style="vertical-align: top; word-wrap: break-word">The file path to the location of the truststore of the client that sends the HTTP events through 'https' protocol. A custom client-truststore can be specified if required.</td>
        <td style="vertical-align: top">${carbon.home}/resources/security/client-truststore.jks</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">client.truststore.password</td>
        <td style="vertical-align: top; word-wrap: break-word">The password for the client-truststore. A custom password can be specified if required. If no custom password is specified and the protocol of URL is 'https' then, the system uses default password.</td>
        <td style="vertical-align: top">wso2carbon</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">headers</td>
        <td style="vertical-align: top; word-wrap: break-word">The headers that should be included as a HTTP request headers. There can be any number of headers concatenated on following format. header1:value1#header2:value2. User can include content-type header if he need to any specific type for payload if not system get the mapping type as the content-Type header (ie. @map(xml):application/xml,@map(json):application/json,@map(text):plain/text ) and if user does not include any mapping type then system gets the 'plain/text' as default Content-Type header. If user does not include Content-Length header then system calculate the bytes size of payload and include it as content-length header.</td>
        <td style="vertical-align: top"> </td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">Yes</td>
    </tr>
    <tr>
        <td style="vertical-align: top">method</td>
        <td style="vertical-align: top; word-wrap: break-word">For HTTP events, HTTP_METHOD header should be included as a request header. If the parameter is null then system uses 'POST' as a default header.</td>
        <td style="vertical-align: top">POST</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">Yes</td>
    </tr>
</table>

<span id="system-parameters" class="md-typeset" style="display: block; font-weight: bold;">System Parameters</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Parameters</th>
    </tr>
    <tr>
        <td style="vertical-align: top">latency.metrics.enabled</td>
        <td style="vertical-align: top; word-wrap: break-word">Property to enable metrics logs to monitor transport latency for config.</td>
        <td style="vertical-align: top">true</td>
        <td style="vertical-align: top">Any Integer</td>
    </tr>
    <tr>
        <td style="vertical-align: top">server.bootstrap.socket.timeout</td>
        <td style="vertical-align: top; word-wrap: break-word">Property to configure specified timeout in milliseconds which server socket will block for this amount of time for http message content to be received.</td>
        <td style="vertical-align: top">15</td>
        <td style="vertical-align: top">Any Integer</td>
    </tr>
    <tr>
        <td style="vertical-align: top">server.bootstrap.boss.group.size</td>
        <td style="vertical-align: top; word-wrap: break-word">Property to configure number of boss threads, which accepts incoming connections until the ports are unbound. Once connection accepts successfully, boss thread passes the accepted channel to one of the worker threads.</td>
        <td style="vertical-align: top">4</td>
        <td style="vertical-align: top">Any integer</td>
    </tr>
    <tr>
        <td style="vertical-align: top">server.bootstrap.worker.group.size</td>
        <td style="vertical-align: top; word-wrap: break-word">Property to configure number of worker threads, which performs non blocking read and write for one or more channels in non-blocking mode.</td>
        <td style="vertical-align: top">8</td>
        <td style="vertical-align: top">Any integer</td>
    </tr>
    <tr>
        <td style="vertical-align: top">default.protocol</td>
        <td style="vertical-align: top; word-wrap: break-word">The default protocol.</td>
        <td style="vertical-align: top">http</td>
        <td style="vertical-align: top">http<br>https</td>
    </tr>
    <tr>
        <td style="vertical-align: top">https.truststore.file</td>
        <td style="vertical-align: top; word-wrap: break-word">The default truststore file path.</td>
        <td style="vertical-align: top">${carbon.home}/resources/security/client-truststore.jks</td>
        <td style="vertical-align: top">Path to client-truststore.jks</td>
    </tr>
    <tr>
        <td style="vertical-align: top">https.truststore.password</td>
        <td style="vertical-align: top; word-wrap: break-word">The default truststore password.</td>
        <td style="vertical-align: top">wso2carbon</td>
        <td style="vertical-align: top">Truststore password</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@sink(type='http',publisher.url='http://localhost:8009/foo', method='{{method}}',headers='{{headers}}', @map(type='xml' , @payload('{{payloadBody}}')))define stream FooStream (payloadBody String, method string, headers string);

```
<p style="word-wrap: break-word">If it is xml mapping expected input should be in following format for FooStream:{&lt;events&gt;    &lt;event&gt;        &lt;symbol&gt;WSO2&lt;/symbol&gt;        &lt;price&gt;55.6&lt;/price&gt;        &lt;volume&gt;100&lt;/volume&gt;    &lt;/event&gt;&lt;/events&gt;,POST,Content-Length:24#Content-Location:USA#Retry-After:120}Above event will generate output as below.~Output http event payload&lt;events&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/event&gt;<br>&lt;/events&gt;<br>~Output http event headersContent-Length:24,Content-Location:'USA',Retry-After:120,Content-Type:'application/xml',HTTP_METHOD:'POST',~Output http event propertiesHTTP_METHOD:'POST',HOST:'localhost',PORT:8009PROTOCOL:'http'TO:'/foo'</p>

